package de.beyondjava.simpleClassParser

import java.util.regex.Matcher
import java.util.regex.Pattern
import static RegularExpressions.*

/**
 * This is a simple bean parser that reads every java or groovy class in a folder
 * and returns a list of managed beans and a list of every variable accessible
 * by the jsf expression language.
 */
class SimpleBeanParser {
	List<ClassDefinition> beans = []
	List<VariableDefinition> vars = []

	/** Parse a file or a folder recursively. */
	public SimpleBeanParser parse(String p_folderOrFilename) {
		List<ClassDefinition> classes = new SimpleClassParser().parse(p_folderOrFilename)
		beans = classes.findAll{ClassDefinition c -> c.annotations.any{ it == "@ManagedBean"  }}
		beans.each({ ClassDefinition c -> vars = vars + appendVariables(beanName(c), c, classes)})
		this
	}

	public String beanName(ClassDefinition p_c)
	{
		if (p_c.className.length()==1) p_c.className[0].toLowerCase();
		else p_c.className[0].toLowerCase() + p_c.className[1..-1]
	}

	public List<VariableDefinition> findVariables(String p_prefix, VariableDefinition p_var, List<ClassDefinition> p_classes)
	{
		VariableDefinition var = p_var.clone()
		var.name = "$p_prefix.${var.name}"

		List<VariableDefinition> result = [var];
		String type = p_var.type;
		String newPrefix = "$p_prefix.${p_var.name}"
		p_classes.each({ ClassDefinition c -> if (type==c.className) result = result + appendVariables(newPrefix, c, p_classes) })
		return result
	}

	private List<VariableDefinition> appendVariables(String p_prefix, ClassDefinition c, List<ClassDefinition> p_classes)
	{
		List<VariableDefinition> result = [];
		c.variables.each{result = result + appendVariables(p_prefix, it, p_classes)}
		result
	}

	private List<VariableDefinition> appendVariables(String newPrefix, VariableDefinition var, List p_classes) {
		findVariables(newPrefix, var, p_classes )
	}


}
